(define-module (ui office-server firsttime ajax)
	:use-module (alterator ajax)
	:use-module (alterator woo)
	:use-module (alterator str)
	:use-module (srfi srfi-13)
	:export (init))

;;; root

(define (update-root)
  (let ((is-auto (form-value "auto")))
    (form-update-visibility
	'("passwd_auto" "generate")
	is-auto)
    (form-update-visibility
	'("passwd_1" "passwd_2")
	(not is-auto))))

(define (read-root)
  (form-update-value-list
    '("passwd_auto")
    (woo-read-first "/root")))

(define (commit-root)
  (woo-write "/root"
	     'commit #t
	     'language (form-value "language")
	     'auto (form-value "auto")
	     'passwd_1 (form-value "passwd_1")
	     'passwd_2 (form-value "passwd_2")
	     'passwd_auto (form-value "passwd_auto")))


;;; iptables

(define (read-iptables)
  (form-update-enum "external_ifaces" (woo-list "/net-iptables/ifaces")))

(define (commit-iptables)
  (woo "firsttime"
       "/net-iptables"
       'external_ifaces (or (form-value "external_ifaces") "")))

;;; CA
(define (read-ca)
  (form-update-value-list '("O") (woo-read-first "/ca/root")))

(define (commit-ca)
  (if (string=? (form-value "server_role") "master")
    (let ((first-lang (car (string-cut (form-value "language") #\;))))
	(woo "create" "/ca/root"
	     'language (form-value "language")
	     'C (string-take-right first-lang 2)
	     'O (form-value "O")
	     'confirm #t)
	(woo-write "/ca/certs"
	           'language (form-value "language")
		   'apply_autoupdate #t
		   'autoupdate #t
		   'time "2:00:00"))))

;;; domain
(define (read-domain)
  (form-update-value-list '("domain" "server_role") (woo-read-first "/net-domain")))

(define (commit-domain)
  (let ((domain (form-value "domain"))
	(server_role (form-value "server_role")))
    (or (not-empty-string? server_role)
	(woo-error (_ "Please select server role")))
    (or (not-empty-string? domain)
	(woo-error (_ "Please define domain name")))
    (woo-write "/net-domain"
	       'server_role server_role
	       'domain domain
	       'language (form-value "language")
	       'runhooks #t)))
;;; locale
(define (update-locale)
  (form-update-cookie "language" (form-value "locale"))
  (form-replace "/office-server/firsttime"))

(define (read-locale)
  (catch/message
    (lambda()
      (form-update-enum  "locale" (woo-list "/login/avail_locale"))
      (form-update-value "locale" (car (string-cut (form-value "language") #\;))))))

;;; common

(define (commit)
  (catch/message
    (lambda()
      (commit-root)
      (commit-ca)
      (commit-iptables)
      (commit-domain)
      (form-replace "/firsttime/finish"))))

(define (init)
    (read-locale)
    (read-ca)
    (read-domain)
    (read-root)
    (read-iptables)
    (form-bind "locale" "change" update-locale)
    (form-bind "auto" "click" update-root)
    (form-bind "generate" "click" read-root)
    (form-bind "apply" "click" commit))
